@@ -8,7 +8,6 @@ from sklearn.model_selection import GridSearchCV
 from sklearn.metrics import mean_squared_error
 from geosoup.common import Handler, Opt
 from geosoup.raster import Raster, np, gdal_array, gdal
-from geosoup.common import Timer
 
 
 __all__ = ['HRFRegressor',
@@ -94,10 +93,103 @@ class _Regressor(object):
             regressor_obj = pickle.load(fileptr)
             return regressor_obj
 
+    def get_training_fit(self,
+                         data=None,
+                         regress_limit=None,
+                         output_type='mean'):
+        """
+        Find out how well the training samples fit the model
+        :param data: Samples() instance
+        :param output_type: Metric to be omputed from the random forest (options: 'mean','median','sd')
+        :param regress_limit: List of upper and lower regression limits for training fit prediction
+        :return: None
+        """
+        if data is None:
+            data = self.data
+
+        if self.fit:
+            sample_predictions = getattr(self,
+                                         'sample_predictions',
+                                         None)
+
+            if callable(sample_predictions):
+                pred = sample_predictions(data.format_data(),
+                                          regress_limit=regress_limit,
+                                          output_type=output_type)
+            else:
+                raise RuntimeError("Instance does not have sample_predictions() method")
+
+            self.training_results['rsq'] = pred['rsq'] * 100.0
+            self.training_results['slope'] = pred['slope']
+            self.training_results['intercept'] = pred['intercept']
+            self.training_results['rmse'] = pred['rmse']
+        else:
+            raise ValueError("Model not initialized with samples")
+
+    @staticmethod
+    def param_grid(param_dict):
+        """
+        Method to make list of parameters based on dictionary with parameter values
+        :param param_dict:  Dictionary of parameter grid values
+        :returns: List of parameter dictionaries
+        """
+        names = list(param_dict.keys())
+        values = list(param_dict.values())
+        grids = np.meshgrid(*values)
+        lists = [np.array(arr).flatten() for arr in grids]
+
+        return list(dict(zip(names, param)) for param in zip(*lists))
+
+    @staticmethod
+    def cv_result(regressor,
+                  samples,
+                  param_dict,
+                  n_folds=5,
+                  regress_limit=None,
+                  output_type='mean',
+                  use_weights=False,
+                  return_summary=True):
+        """
+        Find out how well the training samples fit the model
+        :param regressor: _Regressor child class
+        :param samples: data to get model fit on
+        :param param_dict: Paramters dictionary for _Regressor child class
+        :param n_folds: Number of folds to compute results for
+        :param use_weights: If weights should be used for model fit/training
+        :param return_summary: IF summary should be returned or non-summarized results
+        :param output_type: Metric to be omputed from the random forest (options: 'mean','median','sd')
+        :param regress_limit: List of upper and lower regression limits for training fit prediction
+        :return: None
+        """
+        regressor = regressor(data=samples,
+                              **param_dict)
+
+        folds = samples.make_folds(n_folds)
+
+        results = []
+
+        for trn_data, test_data in folds:
+            regressor.fit_data(trn_data.format_data(),
+                               use_weights=use_weights)
+
+            pred = regressor.sample_predictions(test_data.format_data(),
+                                                regress_limit=regress_limit,
+                                                output_type=output_type)
+            results.append(pred)
+
+        if not return_summary:
+            return results
+        else:
+            return {'rsq_mean': np.mean([result['rsq'] for result in results]),
+                    'rsq_sd': np.std([result['rsq'] for result in results]),
+                    'rmse_mean': np.mean([result['rmse'] for result in results]),
+                    'rmse_sd': np.std([result['rmse'] for result in results]),
+                    'split_rsq': [result['rsq'] for result in results],
+                    'split_rmse': [result['rmse'] for result in results]}
+
     def grid_search_param(self,
                           data,
                           param_dict,
-                          scoring='score',
                           cv_folds=5,
                           n_jobs=1,
                           select=True,
@@ -108,41 +200,39 @@ class _Regressor(object):
         will return a set of parameters with the best model score.
         :param data: dictionary with values (generated using Samples.format_data())
         :param param_dict: Dictionary of parameter grid to search for best score
-        :param scoring: Score metric callable method from the regressor
         :param cv_folds: number of folds to divide the samples in
         :param n_jobs: number of parallel processes to run the grid search
         :param select: if the best set of parameters should be selected
         :param allowed_grad: max allowable percent difference between min and max score
         :param select_perc: percentile to choose for best model selection
         """
+
+        if self.regressor is None:
+            raise RuntimeError('Regressor not defined!')
+
         self.data = data
         self.features = data['feature_names']
         self.label = data['label_name']
 
-        if self.regressor is not None:
+        model = GridSearchCV(self.regressor,
+                             param_dict,
+                             cv=cv_folds,
+                             n_jobs=n_jobs)
 
-            model = GridSearchCV(self.regressor,
-                                 param_dict,
-                                 scoring=scoring,
-                                 cv=cv_folds,
-                                 n_jobs=n_jobs)
-
-            if 'weights' not in data:
-                model.fit(data['features'],
-                          data['labels'])
-            else:
-                model.fit(data['features'],
-                          data['labels'],
-                          sample_weight=data['weights'])
+        if 'weights' not in data:
+            model.fit(data['features'],
+                      data['labels'])
         else:
-            raise RuntimeError('Regressor not defined!')
+            model.fit(data['features'],
+                      data['labels'],
+                      sample_weight=data['weights'])
 
         results = model.cv_results_
 
         params = results['params']
         scores_mean = results['mean_test_score']
         scores_sd = results['std_test_score']
-        ranks = results['rank_test_scores']
+        ranks = results['rank_test_score']
 
         if select:
             grad_cond = (scores_sd.max() - scores_sd.min()) / scores_sd.max() <= allowed_grad
@@ -151,7 +241,7 @@ class _Regressor(object):
                                                                      select_perc,
                                                                      interpolation='nearest'))]
             else:
-                param = params[np.where(ranks == 1)]
+                param = params[ranks.index(1)]
             return param
         else:
             return {'params': params,
@@ -254,7 +344,6 @@ class _Regressor(object):
         return defaults
 
     @staticmethod
-    @Timer.timing
     def regress_raster(regressor,
                        raster_obj,
                        outfile=None,
@@ -471,23 +560,35 @@ class MRegressor(_Regressor):
     def __init__(self,
                  data=None,
                  regressor=None,
-                 intercept=True,
-                 jobs=1,
+                 fit_intercept=True,
+                 n_jobs=1,
                  normalize=False,
                  **kwargs):
-
+        """
+        Instantiate MRegressor class
+        :param data:  Data as Samples() instance
+        :param regressor: Linear regressor
+        :param fit_intercept: Whether to calculate the intercept for this model (default: True)
+        :param n_jobs: The number of jobs to use for the computation
+        :param normalize: If True, the regressors X will be normalized before regression by
+                          subtracting the mean and dividing by the l2-norm
+        :param kwargs: Other Key word arguments
+        """
         super(MRegressor, self).__init__(data,
                                          regressor)
         if self.regressor is None:
             self.regressor = linear_model.LinearRegression(copy_X=True,
-                                                           fit_intercept=intercept,
-                                                           n_jobs=jobs,
+                                                           fit_intercept=fit_intercept,
+                                                           n_jobs=n_jobs,
                                                            normalize=normalize)
 
         self.intercept = self.regressor.intercept_ if hasattr(self.regressor, 'intercept_') else None
         self.coefficients = self.regressor.coef_ if hasattr(self.regressor, 'coef_') else None
 
     def __repr__(self):
+        """
+        Representation of MRegressor instance
+        """
         # gather which attributes exist
         attr_truth = [self.coefficients is not None,
                       self.intercept is not None]
@@ -513,7 +614,6 @@ class MRegressor(_Regressor):
             # if empty return empty
             return "<Multiple Linear Regressor: __empty__>"
 
-    @Timer.timing
     def predict(self,
                 arr,
                 ntile_max=5,
@@ -644,26 +744,6 @@ class MRegressor(_Regressor):
             'intercept': lm['intercept'],
         }
 
-    def get_training_fit(self,
-                         regress_limit=None):
-
-        """
-        Find out how well the training samples fit the model
-        :param regress_limit: List of upper and lower regression limits for training fit prediction
-        :return: None
-        """
-        if self.fit:
-            # predict using held out samples and print to file
-            pred = self.sample_predictions(self.data,
-                                           regress_limit=regress_limit)
-
-            self.training_results['rsq'] = pred['rsq'] * 100.0
-            self.training_results['slope'] = pred['slope']
-            self.training_results['intercept'] = pred['intercept']
-            self.training_results['rmse'] = pred['rmse']
-        else:
-            raise ValueError("Model not initialized with samples")
-
     def get_adjustment_param(self,
                              clip=0.025,
                              data_limits=None,
@@ -700,42 +780,46 @@ class RFRegressor(_Regressor):
     def __init__(self,
                  data=None,
                  regressor=None,
-                 trees=10,
-                 samp_split=2,
-                 samp_leaf=1,
+                 n_estimators=10,
+                 min_samples_split=2,
+                 min_samples_leaf=1,
                  max_depth=None,
-                 max_feat='auto',
+                 max_features='auto',
                  oob_score=False,
                  criterion='mse',
                  n_jobs=1,
                  **kwargs):
         """
         Initialize RF regressor using class parameters
-        :param trees: Number of trees
-        :param samp_split: Minimum number of samples for split
+        :param data: Data as Samples() instance
+        :param regressor: Random forest regressor
+        :param n_estimators: Number of trees
+        :param min_samples_split: min number of data points placed in a node before the node is split
+        :param min_samples_leaf: min number of data points allowed in a leaf node
+        :param max_depth: max number of levels in each decision tree of the random forest
+        :param max_features: max number of features considered for splitting a node
         :param oob_score: (bool) calculate out of bag score
         :param criterion: criterion to be used (default: 'mse', options: 'mse', 'mae')
-        (some parameters are as detailed in
-        http://scikit-learn.org/stable/modules/generated/sklearn.ensemble.RandomForestRegressor.html)
+        :param n_jobs: Number of parallel processes to run the regressor on
         """
 
         super(RFRegressor, self).__init__(data,
                                           regressor)
 
         if self.regressor is None:
-            self.regressor = RandomForestRegressor(n_estimators=trees,
+            self.regressor = RandomForestRegressor(n_estimators=n_estimators,
                                                    max_depth=max_depth,
-                                                   min_samples_split=samp_split,
-                                                   min_samples_leaf=samp_leaf,
-                                                   max_features=max_feat,
+                                                   min_samples_split=min_samples_split,
+                                                   min_samples_leaf=min_samples_leaf,
+                                                   max_features=max_features,
                                                    criterion=criterion,
                                                    oob_score=oob_score,
                                                    n_jobs=n_jobs)
-        self.trees = trees
+        self.n_estimators = n_estimators
         self.max_depth = max_depth
-        self.samp_split = samp_split
-        self.samp_leaf = samp_leaf
-        self.max_feat = max_feat
+        self.min_samples_split = min_samples_split
+        self.min_samples_leaf = min_samples_leaf
+        self.max_features = max_features
         self.oob_score = oob_score
         self.criterion = criterion
         self.n_jobs = n_jobs
@@ -827,10 +911,10 @@ class RFRegressor(_Regressor):
                                            0,
                                            feat_arr)
         else:
-            mask_arr = np.zeros([feat_arr.shape[1]]) + 1
+            mask_arr = np.zeros([feat_arr.shape[0]]) + 1
 
-        out_tile = np.zeros(arr.shape[1])
-        tile_arr = np.zeros([self.trees, arr.shape[1]])
+        out_tile = np.zeros(feat_arr.shape[0])
+        tile_arr = np.zeros([self.n_estimators, feat_arr.shape[0]])
 
         if output_type in ('mean', 'median', 'full'):
 
@@ -1034,7 +1118,6 @@ class RFRegressor(_Regressor):
         else:
             return
 
-    @Timer.timing
     def predict(self,
                 arr,
                 output_type=None,
@@ -1078,7 +1161,10 @@ class RFRegressor(_Regressor):
 
         kwargs.update({'output_type': output_type})
 
-        uncert_dict = kwargs['uncert_dict']
+        if 'uncert_dict' in kwargs:
+            uncert_dict = kwargs['uncert_dict']
+        else:
+            uncert_dict = None
 
         for key, value in kwargs.items():
             if key in ('gain', 'bias', 'upper_limit', 'lower_limit'):
@@ -1173,32 +1259,9 @@ class RFRegressor(_Regressor):
                 out_dict[output_type_elem] = self.predict(data['features'],
                                                           output_type=output_type_elem,
                                                           verbose=verbose)
-
         return out_dict
 
-    def get_training_fit(self,
-                         regress_limit=None,
-                         output_type='mean'):
 
-        """
-        Find out how well the training samples fit the model
-        :param output_type: Metric to be omputed from the random forest (options: 'mean','median','sd')
-        :param regress_limit: List of upper and lower regression limits for training fit prediction
-        :return: None
-        """
-
-        if self.fit:
-            # predict using held out samples and print to file
-            pred = self.sample_predictions(self.data,
-                                           regress_limit=regress_limit,
-                                           output_type=output_type)
-
-            self.training_results['rsq'] = pred['rsq'] * 100.0
-            self.training_results['slope'] = pred['slope']
-            self.training_results['intercept'] = pred['intercept']
-            self.training_results['rmse'] = pred['rmse']
-        else:
-            raise ValueError("Model not initialized with samples")
 
     def get_adjustment_param(self,
                              data_limits=None,
@@ -1282,7 +1345,6 @@ class HRFRegressor(RFRegressor):
             "\n---\nRegressors: \n---\n{}".format('\n'.join(repr_regressor)) + \
             "\n---\n\n"
 
-    @Timer.timing
     def regress_raster(self,
                        raster_obj,
                        outfile=None,
