@@ -1,6 +1,6 @@
 import warnings
 from scipy.stats.stats import pearsonr
-from geosoup.common import Handler, Opt, Sublist, Timer, np
+from geosoup.common import Handler, Opt, Sublist, np
 
 
 __all__ = ['Samples']
@@ -498,11 +498,13 @@ class Samples:
         self.merge_data(samp)
 
     def make_folds(self,
-                   n_folds=5):
+                   n_folds=5,
+                   replace=False):
 
         """
         Make n folds in sample sets
-        :param n_folds:
+        :param n_folds: Number of folds
+        :param replace: If the sample selection is with or without replacement
         :return: list of tuples [(training samp, validation samp)...]
         """
 
@@ -514,18 +516,20 @@ class Samples:
         fold_samples = list()
 
         for fold_samp in nsamp_list:
-            if index_list.shape[0] < (2 * fold_samp):
+            if index_list.shape[0] != fold_samp:
                 val_index = np.random.choice(index_list,
                                              size=fold_samp,
-                                             replace=False)
+                                             replace=replace)
             else:
-                val_index = np.random.choice(index_list,
-                                             size=fold_samp,
-                                             replace=False)
+                val_index = index_list
 
-            trn_index = self.index[~np.in1d(self.index, val_index)]
-            index_list = index_list[~np.in1d(index_list, val_index)]
+            trn_index = self.index[~np.in1d(self.index,
+                                            val_index)]
 
-            fold_samples.append((self.selection(trn_index), self.selection(val_index)))
+            index_list = index_list[~np.in1d(index_list,
+                                             val_index)]
+
+            fold_samples.append((self.selection(trn_index),
+                                 self.selection(val_index)))
 
         return fold_samples
